option solver "minos";
option minos_options "iterations=9999999 major_iter=9999999 superbasics=5000 outlev=0 optimality_tolerance=1.0e-14 feasibility_tolerance=1.0e-14";

model;
param nvar;
param ncon;
param feas;
param comp;

set Nonzeros within {1..nvar, 1..ncon};

param LambdaL {0..ncon};
param LambdaU {0..ncon};
var   Lambda  {i in 0..ncon} >= LambdaL[i], <= LambdaU[i];

param gradf {1..nvar};
param J {Nonzeros};

var s >= 0;

var JL {i in 1..nvar} = sum {(i,j) in Nonzeros} J[i,j]*Lambda[j];

minimize prob: s;

subject to slack1 {i in 1..nvar}:
     -s <= gradf[i] - JL[i];

subject to slack2 {i in 1..nvar}:
      s >= gradf[i] - JL[i];

data params.dat;
data bounds.dat;
data sparse.dat;
data matrix.dat;

solve;

param min1 {1..nvar};
param min2 {1..nvar};
let {i in 1..nvar} min1[i] := abs(gradf[i]) + abs(JL[i]);
let {i in 1..nvar} min2[i] := min(abs(gradf[i] - JL[i]), abs(gradf[i] - JL[i]) / (if (min1[i] > 0) then min1[i] else 1));

printf "COPS feasibility error    : %e\n", feas;
printf "COPS complementarity error: %e\n", comp;
printf "COPS optimality error     : %e\n", max {i in 1..nvar} min2[i];

