# Cam Shape Problem
# Alexander S. Bondarenko - Summer 1998
# COPS 2.0 - September 2000
# COPS 3.0 - November 2002
# COPS 3.1 - March 2004

param n > 0, integer;            # number of discretization points     

param pi := 4*atan(1);           # approximation of pi
param R_v > 0;                   # design parameter related to the valve shape
param R_max;                     # maximum allowed radius of the cam
param R_min;                     # minimum allowed radius of the cam
param alpha;                     # curvature limit parameter

param d_theta = 2*pi/(5*(n+1));  # angle between discretization points 
var r{1..n} <= R_max, >= R_min;  # radius of the cam at discretization points

maximize valve_area: ((pi*R_v)/n)*sum {i in 1..n} r[i];

subject to convexity {i in 2..n-1}: 
  - r[i-1]*r[i] - r[i]*r[i+1] + 2*r[i-1]*r[i+1]*cos(d_theta) <= 0;
subject to convex_edge1:
  - R_min*r[1] - r[1]*r[2] + 2*R_min*r[2]*cos(d_theta) <= 0;
subject to convex_edge2:
  - R_min^2 - R_min*r[1] + 2*R_min*r[1]*cos(d_theta) <= 0;
subject to convex_edge3:
   - r[n-1]*r[n] - r[n]*R_max + 2*r[n-1]*R_max*cos(d_theta) <= 0;
subject to convex_edge4:
   - 2*R_max*r[n] + 2*r[n]^2*cos(d_theta) <= 0;

subject to curvature {i in 1..n-1}:
   -alpha*d_theta <= (r[i+1] - r[i]) <= alpha*d_theta;
subject to curvature_edge1:
    -alpha*d_theta <= (r[1] - R_min) <= alpha*d_theta;
subject to curvature_edge2:
    -alpha*d_theta <= (R_max - r[n]) <= alpha*d_theta;

