# Catalyst Mixing Problem 
# Collocaction formulation 
# COPS 3.0 - November 2002
# COPS 3.1 - March 2004

param nh > 0, integer;		# number of subintervals 

param nc > 0, integer;      	# number of collocation points 
param ne := 2;    	  	# number of differential equations
param rho {1..nc};  		# roots of k-th degree Legendre polynomial

param tf := 1;      		# Final time
param h := tf/nh;   

param bc {1..ne};    		# Boundary conditions for x

param alpha;        		# Smoothing parameter

param fact {j in 0..nc} := if j = 0 then 1 else (prod{i in 1..j} i); 

var u {1..nh,1..nc};
var v {1..nh,1..ne};
var w {1..nh,1..nc,1..ne};

# Evaluation of the piecewise polynomial and derivative at collocation points. 

var pp {i in 1..nh, k in 1..nc, s in 1..ne} =  
  v[i,s] + h*sum {j in 1..nc} w[i,j,s]*(rho[k]^j/fact[j]);

var Dpp {i in 1..nh, k in 1..nc, s in 1..ne} =  
  sum {j in 1..nc} w[i,j,s]*(rho[k]^(j-1)/fact[j-1]);

var ppf {s in 1..ne} =  
  v[nh,s] + h*sum {j in 1..nc} w[nh,j,s]/fact[j];

minimize objective:  -1 + ppf[1] + ppf[2] + 
  (alpha/h)*sum{i in 1..nh-1, j in 1..nc} (u[i+1,j] - u[i,j])^2 ;

subject to u_bounds {i in 1..nh, j in 1..nc}: 0.0 <= u[i,j] <= 1.0;

subject to continuity {i in 1..nh-1, s in 1..ne}:  
  v[i,s] + (sum {j in 1..nc} w[i,j,s]*h/fact[j]) = v[i+1,s];

subject to de1 {i in 1..nh, j in 1..nc}:
  Dpp[i,j,1] = u[i,j]*(10*pp[i,j,2] - pp[i,j,1]);
  
subject to de2 {i in 1..nh, j in 1..nc}:
  Dpp[i,j,2] =   u[i,j]*(pp[i,j,1] - 10*pp[i,j,2]) - (1 - u[i,j])*pp[i,j,2];

subject to b_eqn {s in 1..ne}: v[1,s] = bc[s];

 
