# Flow in a Channel Problem
# Collocation formulation
# Alexander S. Bondarenko - Summer 1998
# COPS 2.0 - September 2000
# COPS 3.0 - November 2002
# COPS 3.1 - March 2004

param nc > 0, integer;               # number of collocation points 
param nd > 0, integer;               # order of the differential equation
param nh > 0, integer;               # number of partition intervals
				    
param rho {1..nc};                   # roots of k-th degree Legendre polynomial
param bc {1..2,1..2};                # boundary conditions
param tf;                            # ODEs defined in [0,tf]
param h := tf/nh;                    # uniform interval length
param t {i in 1..nh+1} := (i-1)*h;   # partition

param fact {j in 0..nc+nd} := if j = 0 then 1 else (prod{i in 1..j} i); 

param R >= 0;                        # Reynolds number

# The collocation approximation u is defined by the parameters v and w.
# uc[i,j] is u evaluated at the collocation points.
# Duc[i,j,s] is the (s-1)-th derivative of u at the collocation points.

var v {i in 1..nh,j in 1..nd};
var w {1..nh,1..nc}; 

var uc {i in 1..nh, j in 1..nc, s in 1..nd} =  
  v[i,s] + h*sum {k in 1..nc} w[i,k]*(rho[j]^k/fact[k]);

var Duc {i in 1..nh, j in 1..nc, s in 1..nd} =  
  sum {k in s..nd} v[i,k]*((rho[j]*h)^(k-s)/fact[k-s]) + h^(nd-s+1)*
  sum {k in 1..nc} w[i,k]*(rho[j]^(k+nd-s)/fact[k+nd-s]);

minimize constant_objective: 1.0;

subject to bc_1: v[1,1] = bc[1,1];
subject to bc_2: v[1,2] = bc[2,1];
subject to bc_3:
  sum {k in 1..nd} v[nh,k]*(h^(k-1)/fact[k-1]) + h^nd*
  sum {k in 1..nc} w[nh,k]/fact[k+nd-1] = bc[1,2];
subject to bc_4:
  sum {k in 2..nd} v[nh,k]*(h^(k-2)/fact[k-2]) + h^(nd-1)*
  sum {k in 1..nc} w[nh,k]/fact[k+nd-2] = bc[2,2];

subject to continuity {i in 1..nh-1, s in 1..nd}:
  sum {k in s..nd} v[i,k]*(h^(k-s)/fact[k-s]) + h^(nd-s+1)*
  sum {k in 1..nc} w[i,k]/fact[k+nd-s] = v[i+1,s];

subject to collocation {i in 1..nh, j in 1..nc}:
  sum {k in 1..nc} w[i,k]*(rho[j]^(k-1)/fact[k-1]) =
  R*(Duc[i,j,2]*Duc[i,j,3] - Duc[i,j,1]*Duc[i,j,4]);
