# Marine Population Dynamics Problem
# Collocation formulation
# Alexander S. Bondarenko - Summer 1998
# COPS 2.0 - September 2000
# COPS 3.0 - November 2002
# COPS 3.1 - March 2004

param nc > 0, integer;               # number of collocation points 
param ne > 0, integer;               # number of differential equations
param nm > 0, integer;               # number of measurements
param nh > 0, integer;               # number of partition intervals
				    
param rho {1..nc};                   # roots of k-th degree Legendre polynomial
param tau {1..nm};                   # times at which observations made
param tf := tau[nm];                 # ODEs defined in [0,tf]
param h := tf/nh;                    # uniform interval length
param t {i in 1..nh+1} := (i-1)*h;   # partition

param fact {j in 0..nc} := if j = 0 then 1 else (prod{i in 1..j} i); 

# itau[i] is the largest integer k with t[k] <= tau[i]

param itau {i in 1..nm} := min(nh,floor(tau[i]/h)+1);    

param z {1..nm,1..ne};               # observations
var g {1..ne-1};                     # growth rates
var m {1..ne};                       # mortality rates

# The collocation approximation u is defined by the parameters v and w.
# uc and Duc are, respectively, u and u' evaluated at the collocation points.

var v {1..nh,1..ne};
var w {1..nh,1..nc,1..ne};

var uc {i in 1..nh, j in 1..nc, s in 1..ne} =  
  v[i,s] + h*sum {k in 1..nc} w[i,k,s]*(rho[j]^k/fact[k]);

var Duc {i in 1..nh, j in 1..nc, s in 1..ne} =  
  sum {k in 1..nc} w[i,k,s]*(rho[j]^(k-1)/fact[k-1]);
 
minimize l2error:
  sum {j in 1..nm} (sum {s in 1..ne}(v[itau[j],s] + (
    sum {k in 1..nc} w[itau[j],k,s]*(tau[j]-t[itau[j]])^k/(fact[k]*h^(k-1))) - z[j,s])^2) ;

subject to g_bounds {s in 1..ne-1}: g[s] >= 0;

subject to m_bounds {s in 1..ne}: m[s] >= 0;

subject to continuity {i in 1..nh-1, s in 1..ne}:  
  v[i,s] + h*sum {j in 1..nc} (w[i,j,s]/fact[j]) = v[i+1,s];

subject to de_1 {i in 1..nh, j in 1..nc}:
  Duc[i,j,1] = -(m[1]+g[1])*uc[i,j,1];

subject to de_s {i in 1..nh, j in 1..nc, s in 2..ne-1}:
  Duc[i,j,s] = g[s-1]*uc[i,j,s-1] - (m[s]+g[s])*uc[i,j,s];

subject to de_ne {i in 1..nh,j in 1..nc}:
  Duc[i,j,ne] = g[ne-1]*uc[i,j,ne-1] - m[ne]*uc[i,j,ne];

