# Isomerization of Alpha-Pinene Problem
# Collocation formulation
# Alexander S. Bondarenko - Summer 1998
# COPS 2.0 - September 2000
# COPS 3.0 - November 2002
# COPS 3.1 - March 2004

param nc > 0, integer;               # number of collocation points 
param ne > 0, integer;               # number of differential equations
param np > 0, integer;               # number of ODE parameters
param nm > 0, integer;               # number of measurements
param nh > 0, integer;               # number of partition intervals
				    
param rho {1..nc};                   # roots of k-th degree Legendre polynomial
param bc {1..ne};                    # boundary conditions
param tau {1..nm};                   # times at which observations made
param tf := tau[nm];                 # ODEs defined in [0,tf]
param h := tf/nh;                    # uniform interval length
param t {i in 1..nh+1} := (i-1)*h;   # partition

param fact {j in 0..nc} := if j = 0 then 1 else (prod{i in 1..j} i); 

# itau[i] is the largest integer k with t[k] <= tau[i]

param itau {i in 1..nm} := min(nh,floor(tau[i]/h)+1);    

param z {1..nm,1..ne};               # observations
var theta {1..np};                   # ODE parameters

# The collocation approximation u is defined by the parameters v and w.
# uc and Duc are, respectively, u and u' evaluated at the collocation points.

var v {1..nh,1..ne};
var w {1..nh,1..nc,1..ne};

var uc {i in 1..nh, j in 1..nc, s in 1..ne} =  
  v[i,s] + h*sum {k in 1..nc} w[i,k,s]*(rho[j]^k/fact[k]);

var Duc {i in 1..nh, j in 1..nc, s in 1..ne} =  
  sum {k in 1..nc} w[i,k,s]*(rho[j]^(k-1)/fact[k-1]);
 
minimize l2error:
  sum {j in 1..nm} (sum {s in 1..ne}(v[itau[j],s] + (
    sum {k in 1..nc} w[itau[j],k,s]*(tau[j]-t[itau[j]])^k/(fact[k]*h^(k-1))) - z[j,s])^2) ;

subject to theta_bounds {i in 1..np}: theta[i] >= 0.0;      

subject to ode_bc {s in 1..ne}:  v[1,s] = bc[s];

subject to continuity {i in 1..nh-1, s in 1..ne}:  
  v[i,s] + h*sum {j in 1..nc} (w[i,j,s]/fact[j]) = v[i+1,s];

subject to de1 {i in 1..nh, j in 1..nc}:
  Duc[i,j,1] = - (theta[1]+theta[2])*uc[i,j,1];

subject to de2 {i in 1..nh, j in 1..nc}:
  Duc[i,j,2] = theta[1]*uc[i,j,1];

subject to de3 {i in 1..nh, j in 1..nc}:
  Duc[i,j,3] = theta[2]*uc[i,j,1] - (theta[3]+theta[4])*uc[i,j,3] + theta[5]*uc[i,j,5];

subject to de4 {i in 1..nh, j in 1..nc}:
  Duc[i,j,4] = theta[3]*uc[i,j,3];

subject to de5 {i in 1..nh, j in 1..nc}:
  Duc[i,j,5] = theta[4]*uc[i,j,3] - theta[5]*uc[i,j,5];

