# Largest-small Polygon Problem
# Alexander S. Bondarenko - Summer 1998
# COPS 2.0 - September 2000
# COPS 3.0 - November 2002
# COPS 3.1 - March 2004

param nv integer > 0;           # number of vertices in the polygon
param pi := 4*atan(1);          # approximation of pi

var r {1..nv};                  # polar radius (distance to fixed vertex)
var theta {1..nv};              # polar angle (measured from fixed direction)

maximize polygon_area: 
  0.5*sum{i in 1..nv-1} r[i+1]*r[i]*sin(theta[i+1] - theta[i]);

subject to r_bounds {i in 1..nv}: 0.0 <= r[i] <= 1.0;

subject to theta_bounds {i in 1..nv}: 0.0 <= theta[i] <= pi;

subject to fix_theta_nv: theta[nv] = pi;
subject to fix_r_nv:     r[nv] = 0.0;

subject to ordered_theta {i in 1..nv-1}: theta[i] <= theta[i+1];

subject to distance {i in 1..nv-1, j in i+1..nv}:
  r[i]^2 + r[j]^2 - 2*r[i]*r[j]*cos(theta[j] - theta[i]) <= 1;

