# Goddard Rocket Problem
# Trapezoidal formulation
# COPS 2.0 - September 2000
# COPS 3.0 - November 2002
# COPS 3.1 - March 2004

param h_0;     # Initial height
param v_0;     # Initial velocity
param m_0;     # Initial mass
param g_0;     # Gravity at the surface

# Parameters for the model.

param T_c;
param h_c;
param v_c;
param m_c;

# Derived parameters.

param c := 0.5*sqrt(g_0*h_0);
param m_f := m_c*m_0;
param D_c := 0.5*v_c*(m_0/g_0);
param T_max := T_c*(m_0*g_0);

param nh;      # Number of intervals in mesh

# Height, velocity, mass and thrust of rocket.

var h {0..nh};
var v {0..nh};
var m {0..nh};
var T {0..nh};
var step;
var t_f = step*nh;

# Drag function.

var D {i in 0..nh} = D_c*(v[i]^2)*exp(-h_c*(h[i]-h_0)/h_0);

# Gravity function.

var g{i in 0..nh} = g_0*(h_0/h[i])^2;

maximize final_velocity: h[nh];

subject to step_eqn: step >= 0;

subject to v_bounds {j in 0..nh}: v[j] >= 0.0;
subject to h_bounds {j in 0..nh}: h[j] >= h_0;
subject to m_bounds {j in 0..nh}: m_f <= m[j] <= m_0;
subject to T_bounds {j in 0..nh}: 0.0 <= T[j] <= T_max;

subject to h_eqn {j in 1..nh}:
   h[j] = h[j-1] + .5*step*(v[j] + v[j-1]);

subject to v_eqn {j in 1..nh}:
   v[j] = v[j-1] + .5*step*((T[j] - D[j] - m[j]*g[j])/m[j]
                 +          (T[j-1] - D[j-1] - m[j-1]*g[j-1])/m[j-1]);

subject to m_eqn {j in 1..nh}:
   m[j] = m[j-1] - .5*step*(T[j] + T[j-1])/c;

# Boundary Conditions

subject to h_ic : h[0] = h_0;
subject to v_ic : v[0] = v_0;
subject to m_ic : m[0] = m_0;
subject to m_fc : m[nh] = m_f;

