# Rocket Steering Problem 
# Trapezoidal formulation
# COPS 2.0 - September 2000
# COPS 3.0 - November 2002
# COPS 3.1 - March 2004

param nh;       # Number of subintervals
param a;        # Magnitude of force.   
param u_min;    # Bounds on the control
param u_max;

param pi := 4*atan(1);

param xs {1..4};      # Initial values for the state constraints.
param xf {2..4};      # Final values for the state constraints.

var u {0..nh};        # control
var x {0..nh,1..4};   # state variables
		      
var tf;               # final time
var h = tf/nh;        # step size

minimize final_time: tf;

subject to h_eqn: h >= 0;

subject to u_bounds {j in 0..nh}: u_min <= u[j] <= u_max;

subject to de1 {i in 0..nh-1}:
	x[i+1,1] = x[i,1] + 0.5*h*(x[i,3] + x[i+1,3]);

subject to de2 {i in 0..nh-1}:
	x[i+1,2] = x[i,2] + 0.5*h*(x[i,4] + x[i+1,4]);

subject to de3 {i in 0..nh-1}:
	x[i+1,3] = x[i,3] + 0.5*h*(a*cos(u[i]) + a*cos(u[i+1]));

subject to de4 {i in 0..nh-1}:
	x[i+1,4] = x[i,4] + 0.5*h*(a*sin(u[i]) + a*sin(u[i+1]));

# Boundary conditions

subject to bcs {j in 1..4}: x[0,j] = xs[j];
subject to bcf {j in 2..4}: x[nh,j] = xf[j];

