# Todd Munson
# COPS 3.1 - March 2004

##############################
#####  Initialization
##############################

param V, integer;       			# Number of vertices
param E, integer;       			# Number of elements
param TOL;					# Minimum area

set VERTICES := {1..V};				# Vertex indices
set ELEMENTS := {1..E};				# Element indices
set COORDS   := {1..3} ordered;			# Spatial coordinates

param TETS{ELEMENTS, 1..4} in VERTICES;		# Tetrahedral elements

##############################
##### Variable declarations
##############################

var x{VERTICES, COORDS};    			# Position of vertices

##############################
##### Defined functions
##############################

var norm{e in ELEMENTS} = sum{i in COORDS} 
  ((1*x[TETS[e,2],i] - x[TETS[e,1],i])^2 +
   (2*x[TETS[e,3],i] - x[TETS[e,2],i] - x[TETS[e,1],i])^2 / 3 +
   (3*x[TETS[e,4],i] - x[TETS[e,3],i] - x[TETS[e,2],i] - x[TETS[e,1],i])^2 / 6);

var area{e in ELEMENTS} = sum{i in COORDS}
  1*(x[TETS[e,2],i] - x[TETS[e,1],i])*
     ((x[TETS[e,3],nextw(i)]-x[TETS[e,1],nextw(i)])*
      (x[TETS[e,4],prevw(i)]-x[TETS[e,1],prevw(i)])- 
      (x[TETS[e,3],prevw(i)]-x[TETS[e,1],prevw(i)])*
      (x[TETS[e,4],nextw(i)]-x[TETS[e,1],nextw(i)])) * sqrt(2);

##############################
##### Optimization problem
##############################

minimize f: sum {e in ELEMENTS} norm[e] / (3*area[e]^(2/3));

subject to feasible {e in ELEMENTS}: area[e] >= TOL;

